#!/usr/bin/env node
var bejesus = require('bejesus-api').bejesus;
var path = require('path');
var fs = require('fs');
var sys = require('sys');

process.argv.shift(); process.argv.shift();

var showUsage = function () {
  console.log(
    "bejesus <command> <param1> <param2>\n" +
    "\n" +
    "Commands are:\n" +
    "bejesus coupon <email address>\n" +
    "bejesus user create <username> <password> <email address> <file containing ssh public key> <coupon code>\n" +
    "bejesus user setup <username> <password>\n" +
//    "bejesus user delete <username> <password>\n" +
//    "bejesus user setpass <username> <password> <new password>\n" +
//    "bejesus user setkey <username> <file containing ssh public key>\n" +
    "The commands below require you to have run 'user setup' before/\n" +
    "bejesus apps list\n" +
    "bejesus app create <app-name> <initial js file>\n" +
    "bejesus app info <app-name>\n" +
    "bejesus app start <app-name>\n" +
    "bejesus app restart <app-name>\n" +
    "bejesus app stop <app-name>\n" +
    "bejesus appnpm install <app-name> <package name>\n" +
    "bejesus appnpm upgrade <app-name> <package name>\n" +
    "bejesus appnpm uninstall <app-name> <package name>"
  );
};

if (process.argv.length < 2) {
  showUsage();
  process.exit(1);
};

if (process.argv[0] === "-h" || process.argv[0] === "--help" || process.argv[0] === "help") {
  showUsage();
  process.exit(0);
};

var config = {
  username: "",
  password: ""
};
var config_file = path.join(process.env.HOME, ".bejesusrc");
try {
  var cf_stat = fs.statSync(config_file);
  if (cf_stat.isFile()) {
    var config_str = fs.readFileSync(config_file);
    var lines = config_str.toString().split("\n");
    for(var i in lines) {
      var line = lines[i];
      if (line.length > 2) {
        var prts = line.split("=");
        config[prts[0]] = prts[1];
      }
    }
  }
} catch (e) {
  // sys.puts(sys.inspect(e));
}

var pad = function (str, len) {
  if (len + 1 >= str.length) {
    str = str + Array(len + 1 - str.length).join(" ");
  }
  return str;
}

var check_config = function () {
  if (config.username == "" || config.password == "") {
    console.log("Error: username and password not set in config.\nPlease run bejesus user setup <username> <password>\n");
    process.exit(2);
  }
};

var action = process.argv.shift();
switch(action) {
  case "coupon":
    var email = process.argv.shift();
    var bej = new bejesus();
    bej.coupon_request(email, function (data) {
      console.log(data.status);
    });
    break;
  case "user":
    var subaction = process.argv.shift();
    switch(subaction) {
      case "create":
        var user = process.argv.shift();
        var pass = process.argv.shift();
        var email = process.argv.shift();
        var rsakey = process.argv.shift();
        var coupon = process.argv.shift();
        var bej = new bejesus();
        if (typeof user == 'undefined' || typeof pass == 'undefined' || typeof email == 'undefined' || typeof rsakey == 'undefined' || typeof coupon == 'undefined') {
          console.log("Invalid arguments.");
          showUsage();
          process.exit(1);
        }
        bej.user_create(user, pass, email, rsakey, coupon, function (data) {
          console.log(data.status);
        });
        break;
      case "setup":
        var user = process.argv.shift();
        var pass = process.argv.shift();
        if (typeof user == 'undefined' || typeof pass == 'undefined') {
          console.log("Invalid arguments.");
          showUsage();
          process.exit(1);
        }
        fs.writeFileSync(config_file, "username=" + user + "\npassword=" + pass + "\n");
        console.log("Config settings saved.");
        break;
/*
      case "delete":
        break;
      case "setpass":
        break;
      case "setkey":
        break;
*/
      default:
        console.log("Invalid action");
        showUsage();
        process.exit(1);
        break;
    }
    break;
  case "apps":
    var subaction = process.argv.shift();
    switch(subaction) {
      case "list":
        check_config();
        var bej = new bejesus(config.username, config.password);
        bej.apps_list(function (data) {
          if (data.length > 0) {
            var c = [15, 6, 91, 13, 11];
            console.log(pad("Name", c[0]) + " " + pad("Port", c[1]) + "" + pad("gitrepo", c[2]) + " " + pad("Start", c[3]) + " " + pad("Running", c[4]));
            for(var i in data) {
              console.log(pad(data[i].name, c[0]) + " " + pad(data[i].port, c[1]) + "  " + pad(data[i].gitrepo, c[2]) + " " + pad(data[i].start, c[3]) + " " + pad(data[i].running, c[4]));
            }
          }
        });
        break;
      default:
        console.log("Invalid action");
        showUsage();
        process.exit(1);
        break;
    }
    break;
  case "app":
    var subaction = process.argv.shift();
    switch(subaction) {
      case "create":
        check_config();
        var appname = process.argv.shift();
        var start = process.argv.shift();
        var bej = new bejesus(config.username, config.password);
        bej.app_create(appname, start, function (data) {
          if (data.status == "success") {
            var c = [15, 6, 91, 13, 11];
            console.log(pad("Name", c[0]) + " " + pad("Port", c[1]) + " " + pad("gitrepo", c[2]) + " " + pad("Start", c[3]) + " " + pad("Running", c[4]));
            console.log(pad(appname, c[0]) + " " + pad(data.port, c[1]) + " " + pad(data.gitrepo, c[2]) + " " + pad(data.start, c[3]) + " " + pad(data.running, c[4]));
          } else {
            console.log(data.status);
          }
        });
        break;
      case "info":
        check_config();
        var appname = process.argv.shift();
        var bej = new bejesus(config.username, config.password);
        bej.app_info(appname, function (data) {
          if (data.status == "success") {
            var c = [15, 6, 91, 13, 11];
            console.log(pad("Name", c[0]) + " " + pad("Port", c[1]) + "" + pad("gitrepo", c[2]) + " " + pad("Start", c[3]) + " " + pad("Running", c[4]));
            console.log(pad(appname, c[0]) + " " + pad(data.port, c[1]) + "  " + pad(data.gitrepo, c[2]) + " " + pad(data.start, c[3]) + " " + pad(data.running, c[4]));
          } else {
            console.log(data.status);
          }
        });
        break;
      case "start":
        check_config();
        var appname = process.argv.shift();
        var bej = new bejesus(config.username, config.password);
        bej.app_start(appname, function (data) {
          if (data.status == "success") {
            console.log("App started.");
          } else {
            console.log(data.status);
          }
        });
        break;
      case "restart":
        check_config();
        var appname = process.argv.shift();
        var bej = new bejesus(config.username, config.password);
        bej.app_restart(appname, function (data) {
          if (data.status == "success") {
            console.log("App restarted.");
          } else {
            console.log(data.status);
          }
        });
        break;
      case "stop":
        check_config();
        var appname = process.argv.shift();
        var bej = new bejesus(config.username, config.password);
        bej.app_stop(appname, function (data) {
          if (data.status == "success") {
            console.log("App stopped.");
          } else {
            console.log(data.status);
          }
        });
        break;
      default:
        console.log("Invalid action");
        showUsage();
        process.exit(1);
        break;
    }
    break;
  case "appnpm":
    var subaction = process.argv.shift();
    switch(subaction) {
      case "install":
        var appname = process.argv.shift();
        var package = process.argv.shift();
        check_config();
        var bej = new bejesus(config.username, config.password);
        bej.appnpm_install(appname, package, function (data) {
          if (data.status == "success") {
            console.log(data.output);
          } else {
            console.log(data.status);
          }
        });
        break;
      case "update":
        var appname = process.argv.shift();
        var package = process.argv.shift();
        check_config();
        var bej = new bejesus(config.username, config.password);
        bej.appnpm_update(appname, package, function (data) {
          if (data.status == "success") {
            console.log(data.output);
          } else {
            console.log(data.status);
          }
        });
        break;
      case "uninstall":
        var appname = process.argv.shift();
        var package = process.argv.shift();
        check_config();
        var bej = new bejesus(config.username, config.password);
        bej.appnpm_uninstall(appname, package, function (data) {
          if (data.status == "success") {
            console.log(data.output);
          } else {
            console.log(data.status);
          }
        });
        break;
      default:
        console.log("Invalid action");
        showUsage();
        process.exit(1);
        break;
    }
    break;
  default:
    showUsage();
    process.exit(1);
    break;
};
